/*****************************************************************************
**                                    / _|
**   __ _ _   _ _ __ ___  _ __ __ _  | |_ ___  ___ ___
**  / _` | | | | '__/ _ \| '__/ _` | |  _/ _ \/ __/ __|
** | (_| | |_| | | | (_) | | | (_| | | || (_) \__ \__ \
**  \__,_|\__,_|_|  \___/|_|  \__,_| |_| \___/|___/___/
**
** Copyright (C) 2018 Aurora Free Open Source Software.
**
** This file is part of the Aurora Free Open Source Software. This
** organization promote free and open source software that you can
** redistribute and/or modify under the terms of the GNU Lesser General
** Public License Version 3 as published by the Free Software Foundation or
** (at your option) any later version approved by the Aurora Free Open Source
** Software Organization. The license is available in the package root path
** as 'LICENSE' file. Please review the following information to ensure the
** GNU Lesser General Public License version 3 requirements will be met:
** https://www.gnu.org/licenses/lgpl.html .
**
** Alternatively, this file may be used under the terms of the GNU General
** Public License version 3 or later as published by the Free Software
** Foundation. Please review the following information to ensure the GNU
** General Public License requirements will be met:
** http://www.gnu.org/licenses/gpl-3.0.html.
**
** NOTE: All products, services or anything associated to trademarks and
** service marks used or referenced on this file are the property of their
** respective companies/owners or its subsidiaries. Other names and brands
** may be claimed as the property of others.
**
** For more info about intellectual property visit: aurorafoss.org or
** directly send an email to: contact (at) aurorafoss.org .
*****************************************************************************/

#ifndef AURORAFW_CORELIB_TYPEINFO_H
#define AURORAFW_CORELIB_TYPEINFO_H

#include <AuroraFW/Global.h>
#if(AFW_TARGET_PRAGMA_ONCE_SUPPORT)
	#pragma once
#endif

#include <AuroraFW/Internal/Config.h>

#include <AuroraFW/CoreLib/TypeTraits.h>

namespace AuroraFW {
	enum class TypeID : unsigned
	{
		Unsigned,
		UnsignedInt = Unsigned,
		UnsignedChar,
		Byte = UnsignedChar,
		UnsignedByte = Byte,
		Float,
		Double,
		Real,
		LongDouble = Real,
		Int,
		Int8,
		Int16,
		Int32 = Int,
		Char,
		Long,
		LongInt = Long,
		LongLong,
		LongLongInt = LongLong,
		Bool
	};

	class AFW_API TypeInfo {
		inline static constexpr size_t _sizeOf(TypeID type)
		{
			switch (type)
			{
				case TypeID::Float: return sizeof(float);
				case TypeID::UnsignedInt: return sizeof(unsigned int);
				case TypeID::UnsignedByte: return sizeof(byte_t);
				case TypeID::Bool: return sizeof(bool);
				default: return AFW_NULLVAL;
			}
		}

	public:
		AFW_FORCE_INLINE static constexpr size_t sizeOf(unsigned type)
		{
			return sizeOf(static_cast<TypeID>(type));
		}

		AFW_FORCE_INLINE static constexpr size_t sizeOf(TypeID type)
		{
			return ((AFW_IS_CONSTEXPR(type)) ? static_sizeOf(type) : _sizeOf(type));
		}

		AFW_FORCE_INLINE static constexpr size_t static_sizeOf(TypeID type)
		{
			static_assert(AFW_IS_CONSTEXPR(type));
			return _sizeOf(type);
		}
		AFW_FORCE_INLINE static constexpr size_t static_sizeOf(unsigned type) { return sizeOf(static_cast<TypeID>(type)); }
	};
}

#endif // AURORAFW_CORELIB_TYPEINFO_H